#ifndef _TMAGAME_CPP_
#define _TMAGAME_CPP_

#include "ma_classes.h"
#include <ctype.h>
#include <string.h>

// **************************************
// Definicin de mtodos de clase TMAGame
// **************************************
//
REGISTER_CLASS_ID(TMAGame);

PMAGame TMAGame::oc_MAGame=NULL;
bool TMAGame::KeyState[TMAGame::KEY_MAX];
bool TMAGame::ci_endgame=false;
bool TMAGame::ib_soundAvailable=false;
bool TMAGame::ib_joystickAvailable=false;
long TMAGame::il_time_millis=0;
int  TMAGame::ii_last_ascii_key=0;
int  TMAGame::ARGC=0;
char **TMAGame::ARGV=NULL;
SDL_Surface *TMAGame::ScreenSurf=NULL;
PMABitmap TMAGame::Screen=NULL;

TMAGame::TMAGame(void)
{
    if (oc_MAGame!=NULL)
    {
        exit(0);
    }
    oc_MAGame = this;
    Screen = NULL;    
    ScreenSurf = NULL;    
    io_current_scene=NULL;
    io_next_scene=NULL;
    ib_remove_last_scene=false;
    releaseKeys();
    setGameFPS(0);
}

TMAGame::~TMAGame()
{
    cleanup();
}

void TMAGame::setupLanguage(const char *domain,const char *locale)
{
   /*
   // Obtenemos el directorio del repositorio de idioma
   //
   char buff[1024];
   memset(buff,0,sizeof(buff));

   if (locale!=NULL && *locale!=0)
   {
       #ifdef LINUX
           sprintf(buff,"%s_%c%c.UTF8",locale,toupper(locale[0]),toupper(locale[1]));
           setenv("LANG",buff,1);
           setenv("LANGUAGE",buff,1);
       #else
           sprintf(buff,"LANG=%s_%c%c.UTF8",locale,toupper(locale[0]),toupper(locale[1]));
           putenv(buff);
       #endif
   }
   getcwd(buff,1024);
   strcat(buff,"/data/lang");
   for (int i=0;*(buff+i)!=0;i++) if (*(buff+i)=='\\') *(buff+i)='/';

   // Establecemos el idioma
   //
   set_uformat(U_UTF8);
   setlocale(LC_MESSAGES,"");
   bindtextdomain(domain,buff);
   textdomain(domain);
   */
}

void TMAGame::releaseKeys(void)
{
    memset(KeyState,0,sizeof(KeyState));
    releaseLastAsciiKeyPressed();
}

void TMAGame::setGameFPS(int fps)
{
    ii_game_speed = (fps<=0)?0:(1000/fps);
}

PMAGame TMAGame::getCurrentGame(void)
{
    return oc_MAGame;
}

long TMAGame::getCPUTime(void)
{
    return SDL_GetTicks();
}

void TMAGame::sleep(int ms)
{
   SDL_Delay(ms);
}

void TMAGame::showAlert(const char *txt)
{
    //
}

void TMAGame::showLastAlert()
{
    //
}

void TMAGame::showFatalError(const char *txt)
{
    //
    endGame();
}

void TMAGame::setWindowTitle(const char *txt)
{
    #ifndef TARGET_GP2X
        SDL_WM_SetCaption(txt,NULL);
    #endif
}

void TMAGame::getScreenSize(int& width,int& height)
{
    SDL_Surface *lscr = (ScreenSurf!=NULL)?ScreenSurf:SDL_GetVideoSurface();

    width = lscr->w;
    height = lscr->h;
}

int TMAGame::getScreenWidth(void)
{
    SDL_Surface *lscr = (ScreenSurf!=NULL)?ScreenSurf:SDL_GetVideoSurface();
    return lscr->w;
}

int TMAGame::getScreenHeight(void)
{
    SDL_Surface *lscr = (ScreenSurf!=NULL)?ScreenSurf:SDL_GetVideoSurface();
    return lscr->h;
}



void TMAGame::FlipScreen(void)
{
    SDL_Flip(ScreenSurf);
}
    
bool TMAGame::setGraphicsMode(int width,int height,int colordepth,bool fullscreen)
{
    Uint32 flags =  SDL_HWSURFACE|SDL_DOUBLEBUF;
    if (fullscreen) flags|=SDL_FULLSCREEN;
    
    if (Screen!=NULL)
    {
        delete Screen;
        Screen=NULL;
        SDL_FreeSurface(ScreenSurf);
    }
    
    ScreenSurf = SDL_SetVideoMode(width, height, colordepth, flags);
    if (ScreenSurf!=NULL)
    {
        if (Screen!=NULL) delete Screen;
        Screen = new TMABitmap(ScreenSurf,true);
    }
    return (ScreenSurf!=NULL);
}

int TMAGame::getScreenDepth(void)  { return ScreenSurf->format->BitsPerPixel; }
        
bool TMAGame::isFullScreen()
{
    if (Screen==NULL) return false;
    return (ScreenSurf->flags & SDL_FULLSCREEN);
}

bool TMAGame::setFullScreen(bool fs)
{
    return setGraphicsMode(getScreenWidth(),getScreenHeight(),getScreenDepth(),fs);
}
        
void TMAGame::cleanup(void)
{    
	if (Screen != NULL) 
    {
        delete Screen;
        Screen = NULL;
    }
}

void TMAGame::endGame(void)
{
    ci_endgame = true;
}

void TMAGame::setGP2XKey(int keyp,bool down)
{
    int key=0;
    
    switch (keyp)
    {
        case GP2X_BUTTON_L:
            key = SDLK_LSHIFT;
            break;
            
        case GP2X_BUTTON_R:
            key = SDLK_RSHIFT;
            break;
            
        case GP2X_BUTTON_UP:
            key = SDLK_UP;
            break;
            
        case GP2X_BUTTON_LEFT:
            key = SDLK_LEFT;
            break;
            
        case GP2X_BUTTON_RIGHT:
            key = SDLK_RIGHT;
            break;
            
        case GP2X_BUTTON_DOWN:
            key = SDLK_DOWN;
            break;
            
        case GP2X_BUTTON_A:
            key = SDLK_a;
            break;

        case GP2X_BUTTON_B:
            key = SDLK_b;
            break;

        case GP2X_BUTTON_Y:
            key = SDLK_y;
            break;

        case GP2X_BUTTON_X:
            key = SDLK_x;
            break;
            
        case GP2X_BUTTON_VOLDOWN:
            key = SDLK_LESS;
            break;
            
        case GP2X_BUTTON_VOLUP:
            key = SDLK_GREATER;
            break;
            
        case GP2X_BUTTON_SELECT:
            key = SDLK_RETURN;
            break;
            
        case GP2X_BUTTON_HOME:
            key = SDLK_ESCAPE;
            break;
    }
    
    if (key>0) KeyState[key]=down;
}

int TMAGame::getLastAsciiKeyPressed(void)
{
    return ii_last_ascii_key;
}

void TMAGame::releaseLastAsciiKeyPressed(void)
{
    ii_last_ascii_key=0;
}

void TMAGame::setVolume(int v)
{
    Mix_Volume(-1,v);
    Mix_VolumeMusic(v);
}

void TMAGame::run(void)
{    
    long startTime=0,cur_wait ;
    SDL_Event event;
    SDL_ShowCursor(SDL_DISABLE);
    
    // ---- CICLO PRINCIPAL DEL JUEGO
    //
    while (!ci_endgame)
    {
        if (ii_game_speed) startTime = getCPUTime() + ii_game_speed;
        
        // Invocamos el RUN
        //
        if ( (io_current_scene==NULL) || (!io_current_scene->doRun()) ) doRun();
        
        if (io_next_scene==NULL)
        {
            // Invocamos el Paint
            //
            if (!io_current_scene) doPaint(*Screen);
            else io_current_scene->doPaint(*Screen);
            FlipScreen();
        }
        else
        {
            // Establecemos la nueva escena, si se cambio.
            //
            if (ib_remove_last_scene && io_current_scene!=NULL) delete io_current_scene;
            io_current_scene = io_next_scene;
            io_next_scene=NULL;
        }

        while (SDL_PollEvent (&event))
        {
            switch (event.type)
            {
                #ifndef TARGET_GP2X
                    case SDL_KEYDOWN:
                        KeyState[event.key.keysym.sym]=true;
                        ii_last_ascii_key = event.key.keysym.unicode;
                        break;                        
                    case SDL_KEYUP:
                        KeyState[event.key.keysym.sym]=false;
                        releaseLastAsciiKeyPressed();
                        break;
                #endif
                
                case SDL_JOYBUTTONDOWN:
                    #ifdef TARGET_GP2X
                        setGP2XKey(event.jbutton.button,true);
                    #endif
                    break;
                    
                case SDL_JOYBUTTONUP:
                    #ifdef TARGET_GP2X
                        setGP2XKey(event.jbutton.button,false);
                    #endif
                    break;

                case SDL_QUIT:
                    endGame();
                    break;
            }
        }
        
        // Duerme el tiempo necesario.
        //
        if (ii_game_speed>0)
        {
            cur_wait = startTime - getCPUTime();
            if (cur_wait>0) sleep(cur_wait);
        }
    }
    //
    // ----------- FIN DE CICLO PRINCIPAL 
}

bool TMAGame::setScene(PMAScene esc,const PMAObject params,bool remove_last)
{
    if (io_next_scene!=NULL)
    {
        delete io_next_scene;
        io_next_scene=NULL;
    }
    
    if (!esc->isInitialized())
    {
        if (!esc->doInit(params)) return false;
        io_next_scene = esc;
        ib_remove_last_scene=remove_last;
    }
    
    return true;
}

// ***************************************
// Definicin de mtodos de clase TMAScene
// ***************************************
//
REGISTER_CLASS_ID(TMAScene);

TMAScene::TMAScene()
{
    ib_initialized=false;
    ii_Estado=0;
}

bool TMAScene::doInit(const PMAObject params)
{
    ib_initialized=true;
    return true;
}

#endif
