#ifndef _GLAURUNG_CPP_
#define _GLAURUNG_CPP_

#include "profanation.h"
#include "../escenas/escenarios.h"

// **************************************
// Definicin de mtodos de clase TMAGame
// **************************************
//
REGISTER_CLASS_ID(TProfanationGame);

TMAString TProfanationGame::gs_charset("ABCDEFGHIJKLMNOPQRSTUVWXYZ:.,()*0123456789 '-");

#ifdef TARGET_GP2X
    int TProfanationGame::ga_GameKeys[10]={SDLK_LEFT,SDLK_RIGHT,SDLK_y,SDLK_x,SDLK_ESCAPE,SDLK_RETURN,SDLK_UP,SDLK_DOWN,SDLK_GREATER,SDLK_LESS};
#else
    int TProfanationGame::ga_GameKeys[10]={SDLK_LEFT,SDLK_RIGHT,SDLK_UP,SDLK_DOWN,SDLK_ESCAPE,SDLK_RETURN,SDLK_UP,SDLK_DOWN,SDLK_PAGEUP,SDLK_PAGEDOWN};
#endif

int TProfanationGame::gi_idioma=TProfanationGame::LG_SPAIN;
int TProfanationGame::gi_idioma_anterior=TProfanationGame::LG_SPAIN;
int TProfanationGame::gi_volume=128;

// --------- IMPLEMENTACION DE DESTRUCTOR
//
TProfanationGame::~TProfanationGame()
{
    cleanup();
}

void TProfanationGame::cleanup(void) {}

void TProfanationGame::showFullScreen(bool fs)
{
    config_fullscreen = fs;    
    TMAGame::setFullScreen(fs);
    saveConfiguration();
}

void TProfanationGame::increaseVolume()
{
    if (TProfanationGame::gi_volume<128) 
    {
        TProfanationGame::gi_volume++;
        saveConfiguration();
        setVolume(TProfanationGame::gi_volume);
    }
}

void TProfanationGame::decreaseVolume()
{
    if (TProfanationGame::gi_volume>0) 
    {
        TProfanationGame::gi_volume--;
        saveConfiguration();
        setVolume(TProfanationGame::gi_volume);
    }
}

// --------- INICIALIZACION DEL JUEGO
//
bool TProfanationGame::init(void)
{ 
    lb_init=false; 
     
    if (!TMAGame::init()) return false;
    loadConfiguration();
    
    if (!setGraphicsMode(640/_REDUCE,480/_REDUCE,16,config_fullscreen)) return false;
    setWindowTitle("Abusimbel Profanation Deluxe");
    setGameFPS(32);
    
    lb_init = true;
                
    // Cargamos los tipos de letras constantes en el juego
    //
    #ifdef SMALL_GAME
        fontMulti.load("Data/Fonts/FuenteMulti.png",10,15,gs_charset.length(),1,false,0);
        fontBlue.load("Data/Fonts/FuenteBlue.png",10,15,gs_charset.length(),1,false,0);
    #else
        fontMulti.load("Data/Fonts/FuenteMulti.png",20,30,gs_charset.length(),1,false,MAKERGBCOLOR(255,0,255));
        fontBlue.load("Data/Fonts/FuenteBlue.png",20,30,gs_charset.length(),1,false,MAKERGBCOLOR(255,0,255));
    #endif
    fontMulti.setCharacters(*gs_charset);
    fontBlue.setCharacters(*gs_charset);
    
    // Cargamos los sonidos
    //
    if (isSoundAvailable())
    {
        setVolume(gi_volume);
        gsnd_Gota.load("Data/Sonidos/gota.wav");
        gsnd_Roca.load("Data/Sonidos/roca.wav");
        gsnd_Camina.load("Data/Sonidos/camina.wav");
        gsnd_Salto.load("Data/Sonidos/salto.wav");
        gsnd_Caida.load("Data/Sonidos/caida.wav");
        gsnd_Tesoro.load("Data/Sonidos/tesoro.wav");
        gsnd_Scroll.load("Data/Sonidos/scroll.wav");
        gsnd_Teletransporte.load("Data/Sonidos/teletransporte.wav");
        gsnd_Grito.load("Data/Sonidos/grito.wav");
    }
    
    miRND.init();
    
    // Establecemos el escenario inicial
    //    
    setScene(new TESC_Seleccion_Idioma());
    
    return true;
}


bool TProfanationGame::testKey(int pkey) 
{ 
    return KeyState[ga_GameKeys[pkey]]; 
}

void TProfanationGame::releaseKey(int pkey) 
{ 
    KeyState[ga_GameKeys[pkey]]=false; 
}

bool TProfanationGame::isKeyPressed(void)
{
    for (unsigned int i=0;i<sizeof(ga_GameKeys);i++)
    {
        if (KeyState[ga_GameKeys[i]]) return true;
    }
    return false;
}

void TProfanationGame::doRun(void)
{
    if (testKey(TProfanationGame::GKEY_ABORT))
    {
        this->endGame();
        return;
    }
    
    if (testKey(TProfanationGame::GKEY_VOLUME_DOWN)) decreaseVolume();
    if (testKey(TProfanationGame::GKEY_VOLUME_UP)) increaseVolume();
}

void TProfanationGame::doPaint(TMABitmap& buff)
{
    buff.clear(0);
};

void TProfanationGame::loadConfiguration(void)
{
 config_fullscreen = false;
 
 // Intentamos abrir el archivo de configuracion
 //
 FILE *f = fopen("Data/config","rb");
 
 // Si no existe, intentamos crear uno con los valores actuales.
 //
 if (!f)
 {
  saveConfiguration();
  return;
 }
 
 // Leemos los valores.
 //
 fread(&config_fullscreen,sizeof(config_fullscreen),1,f);
 fread(&TProfanationGame::gi_idioma,sizeof(TProfanationGame::gi_idioma),1,f);
 fread(&TProfanationGame::gi_idioma_anterior,sizeof(TProfanationGame::gi_idioma_anterior),1,f);
 fread(&TProfanationGame::ga_GameKeys[0],sizeof(TProfanationGame::ga_GameKeys),1,f);
 fread(&TProfanationGame::gi_volume,sizeof(TProfanationGame::gi_volume),1,f);
 fclose(f);
}

void TProfanationGame::saveConfiguration(void)
{
 // Intentamos crear el archivo de configuracion
 //
 FILE *f = fopen("Data/config","wb");
 
 // Si no podemos crearlo, desistimos
 //
 if (!f) return;
 
 // Escribimos los valores.
 //
 config_fullscreen = (lb_init)?this->isFullScreen():false;
 fwrite(&config_fullscreen,sizeof(config_fullscreen),1,f);
 fwrite(&TProfanationGame::gi_idioma,sizeof(TProfanationGame::gi_idioma),1,f);
 fwrite(&TProfanationGame::gi_idioma_anterior,sizeof(TProfanationGame::gi_idioma_anterior),1,f);
 fwrite(&TProfanationGame::ga_GameKeys[0],sizeof(TProfanationGame::ga_GameKeys),1,f);
 fwrite(&TProfanationGame::gi_volume,sizeof(TProfanationGame::gi_volume),1,f);
 fclose(f);
}

int MainProc(int argc, char** argv)
{ 
    TProfanationGame miGame;
    
    if (miGame.init())
    {
        miGame.run();
        
        return 0;
    }
    else
    {
        miGame.showAlert("No se pudo inicializar la aplicacin.");
    }
    return -1;    
}

#endif
